package com.tzld.longarticle.recommend.server.remote.aigc;

import com.alibaba.fastjson.JSONObject;
import com.tzld.longarticle.recommend.server.model.vo.aigc.ProducePlanDetailVO;
import com.tzld.longarticle.recommend.server.util.HttpClientFactory;
import lombok.extern.slf4j.Slf4j;
import org.apache.http.HttpEntity;
import org.apache.http.StatusLine;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.util.EntityUtils;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.util.Objects;

@Service
@Slf4j
public class AIGCProducePlanDetailService {

    private final CloseableHttpClient client =
            HttpClientFactory.create(10000, 180000, 200, 200, 0, 10000);

    public ProducePlanDetailVO articleGetProducePlanDetail(String planId) {
        int retryTimes = 2;
        while (retryTimes > 0) {
            ProducePlanDetailVO result = post(planId);
            if (Objects.nonNull(result)) {
                return result;
            }
            retryTimes--;
            try {
                Thread.sleep(1000);
            } catch (InterruptedException ignored) {
            }
        }
        return null;
    }

    public ProducePlanDetailVO post(String planId) {
        String apiUrl = "http://aigc-api.cybertogether.net/aigc/produce/plan/detail";

        // 构建请求的 JSON Payload
        JSONObject payload = new JSONObject();
        JSONObject params = new JSONObject();
        params.put("id", planId);
        payload.put("params", params);
        payload.put("baseInfo", getBaseInfo());
        try {
            HttpPost httpPost = new HttpPost(apiUrl);
            StringEntity stringEntity = new StringEntity(payload.toJSONString(), StandardCharsets.UTF_8);
            httpPost.setHeader("Content-Type", "application/json;charset=UTF-8");
            httpPost.setEntity(stringEntity);
            CloseableHttpResponse response = client.execute(httpPost);
            StatusLine statusLine = response.getStatusLine();
            if (statusLine.getStatusCode() == 200) {
                HttpEntity responseEntity = response.getEntity();
                if (Objects.nonNull(responseEntity)) {
                    String responseBody = EntityUtils.toString(responseEntity, "UTF-8");
                    JSONObject jsonObject = JSONObject.parseObject(responseBody);
                    if (jsonObject.getInteger("code") == 0) {
                        return jsonObject.getObject("data", ProducePlanDetailVO.class);
                    }
                }
            }
        } catch (IOException e) {
            log.error("articleGetProducePlanDetail error", e);
        }
        return null;
    }

    private JSONObject getBaseInfo() {
        JSONObject baseInfo = new JSONObject();
        baseInfo.put("token", "79dcb40af1de43b1bd9fe3a731dbaa15");
        baseInfo.put("appType", 9);
        baseInfo.put("platform", "pc");
        baseInfo.put("appVersionCode", 1000);
        baseInfo.put("clientTimestamp", 1);
        baseInfo.put("fid", 1);
        baseInfo.put("loginUid", 1);
        baseInfo.put("pageSource", 1);
        baseInfo.put("requestId", 1);
        baseInfo.put("rid", 1);
        baseInfo.put("uid", 1);
        return baseInfo;
    }
}
