package com.tzld.longarticle.recommend.server.remote.aigc;

import com.alibaba.fastjson.JSONObject;
import com.tzld.longarticle.recommend.server.model.param.LongArticleSystemFilterSortParam;
import lombok.extern.slf4j.Slf4j;
import okhttp3.*;
import org.springframework.stereotype.Service;

import javax.annotation.PostConstruct;
import java.util.concurrent.TimeUnit;

@Service
@Slf4j
public class AIGCPublishContentDiscardService {

    private OkHttpClient client;

    @PostConstruct
    public void init() {
        client = new OkHttpClient().newBuilder()
                .connectionPool(new ConnectionPool(20, 5, TimeUnit.MINUTES))
                .connectTimeout(5, TimeUnit.MINUTES)
                .readTimeout(5, TimeUnit.MINUTES)
                .writeTimeout(5, TimeUnit.MINUTES)
                .build();
    }

    private static final String filterSortContentsUrl = "http://aigc-api.cybertogether.net/aigc/publish/LongArticleSystem/filterSortContents";


    public Boolean filterSortContents(LongArticleSystemFilterSortParam param) {
        int retryTimes = 2;
        while (retryTimes > 0) {
            Boolean result = post(param);
            if (result) {
                return true;
            }
            retryTimes--;
        }
        return false;
    }

    public Boolean post(LongArticleSystemFilterSortParam param) {
        Boolean result = false;
        JSONObject bodyParam = new JSONObject();
        bodyParam.put("params", param);
        try {
            MediaType mediaType = MediaType.parse("application/json");
            RequestBody body = RequestBody.create(mediaType, JSONObject.toJSONString(bodyParam));
            Request request = new Request.Builder()
                    .url(filterSortContentsUrl)
                    .method("POST", body)
                    .addHeader("Content-Type", "application/json;charset=UTF-8")
                    .build();
            Response response = client.newCall(request).execute();
            if (response.isSuccessful()) {
                String responseContent = response.body().string();
                JSONObject jsonObject = JSONObject.parseObject(responseContent);
                if (jsonObject.getInteger("code") == 0) {
                    result = true;
                }
            }
        } catch (Exception e) {
            log.error("getAllContent error", e);
        }
        return result;
    }
}
