package com.tzld.longarticle.recommend.server.remote.aigc;

import cn.hutool.core.collection.CollectionUtil;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.tzld.longarticle.recommend.server.model.dto.Content;
import com.tzld.longarticle.recommend.server.model.entity.longArticle.PublishContentGzhWaiting;
import com.tzld.longarticle.recommend.server.repository.longArticle.PublishContentGzhWaitingRepository;
import com.tzld.longarticle.recommend.server.service.XxlJobService;
import lombok.extern.slf4j.Slf4j;
import okhttp3.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.PostConstruct;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.TimeUnit;

/**
 * @author dyp
 */
@Service
@Slf4j
public class AIGCWaitingPublishContentService {

    @Autowired
    private PublishContentGzhWaitingRepository publishContentGzhWaitingRepository;

    @Autowired
    private XxlJobService xxlJobService;

    private OkHttpClient client;

    @PostConstruct
    public void init() {
        client = new OkHttpClient().newBuilder()
                .connectionPool(new ConnectionPool(20, 5, TimeUnit.MINUTES))
                .connectTimeout(5, TimeUnit.MINUTES)
                .readTimeout(5, TimeUnit.MINUTES)
                .writeTimeout(5, TimeUnit.MINUTES)
                .build();
    }

    private static final String aigcGetAllContentUrl = "http://aigc-api.cybertogether.net/aigc/publish/content/gzhWaitingPublishContent";


    public List<Content> getAllContent(String planId, String accountId, List<String> sourceIds) {
        int retryTimes = 2;
        while (retryTimes > 0) {
            List<Content> result = post(planId, accountId, sourceIds);
            if (Objects.nonNull(result)) {
                return result;
            }
            retryTimes--;
        }
        return new ArrayList<>();
    }

    public List<Content> post(String planId, String accountId, List<String> sourceIds) {
        List<Content> result = null;
        JSONObject bodyParam = new JSONObject();
        JSONObject bodyParamParams = new JSONObject();
        bodyParamParams.put("accountId", accountId);
        bodyParamParams.put("planId", planId);
        bodyParamParams.put("sourceIds", sourceIds);
        bodyParam.put("params", bodyParamParams);
        try {
            MediaType mediaType = MediaType.parse("application/json");
            RequestBody body = RequestBody.create(mediaType, JSONObject.toJSONString(bodyParam));
            Request request = new Request.Builder()
                    .url(aigcGetAllContentUrl)
                    .method("POST", body)
                    .addHeader("Content-Type", "application/json;charset=UTF-8")
                    .build();
            Response response = client.newCall(request).execute();
            if (response.isSuccessful()) {
                String responseContent = response.body().string();
                JSONObject jsonObject = JSONObject.parseObject(responseContent);
                if (jsonObject.getInteger("code") == 0) {
                    JSONArray data = jsonObject.getJSONArray("data");
                    result = JSONArray.parseArray(data.toJSONString(), Content.class);
                }
            }
        } catch (Exception e) {
            log.error("getAllContent error", e);
        }
        return result;
    }

    public List<Content> getAllContentByDB(String planId, String accountId) {
        List<PublishContentGzhWaiting> list = publishContentGzhWaitingRepository.getByPlanIdAndPublishAccountIdAndStatus(planId, accountId, 1);
        if (CollectionUtil.isEmpty(list)) {
            List<Content> contentList = getAllContent(planId, accountId, null);
            if (CollectionUtil.isNotEmpty(contentList)) {
                Long now = System.currentTimeMillis();
                CompletableFuture.runAsync(() -> xxlJobService.savePublishContentCache(contentList, planId, accountId, true, now));
            }
            return contentList;
        }
        List<Content> result = new ArrayList<>();
        for (PublishContentGzhWaiting item : list) {
            Content content = new Content();
            content.setId(item.getId());
            content.setSourceType(item.getSourceType());
            content.setSourceId(item.getSourceId());
            content.setTitle(item.getTitle());
            content.setCreateTimestamp(item.getCreateTimestamp());
            content.setContentPoolType(item.getContentPoolType());
            content.setCrawlerChannelContentId(item.getCrawlerChannelContentId());
            content.setCrawlerLink(item.getCrawlerLink());
            content.setCrawlerTitle(item.getCrawlerTitle());
            content.setCrawlerViewCount(item.getCrawlerViewCount());
            content.setCrawlerTimestamp(item.getCrawlerTimestamp());
            result.add(content);
        }
        return result;
    }

}
