package com.tzld.longarticle.recommend.server.service;

import com.tzld.longarticle.recommend.server.mapper.crawler.ArticleGzhDeveloperMapper;
import com.tzld.longarticle.recommend.server.mapper.crawler.ArticleUserGroupMapper;
import com.tzld.longarticle.recommend.server.mapper.crawler.ext.ArticleUserGroupMapperExt;
import com.tzld.longarticle.recommend.server.model.dto.UserGroupCountDTO;
import com.tzld.longarticle.recommend.server.model.vo.WxUserGetResponse;
import com.tzld.longarticle.recommend.server.remote.WxUserManagementRemoteService;
import com.tzld.longarticle.recommend.server.repository.model.ArticleGzhDeveloper;
import com.tzld.longarticle.recommend.server.repository.model.ArticleUserGroup;
import com.tzld.longarticle.recommend.server.repository.model.ArticleUserGroupExample;
import com.tzld.longarticle.recommend.server.util.CommonCollectionUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang.math.RandomUtils;
import org.apache.commons.lang3.StringUtils;
import org.dom4j.Document;
import org.dom4j.Element;
import org.dom4j.io.SAXReader;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.io.StringReader;
import java.util.*;

/**
 * @author dyp
 */
@Slf4j
@Service
public class UserManagementService {
    @Autowired
    private WxUserManagementRemoteService wxUserManagementRemoteService;

    @Autowired
    private ArticleUserGroupMapper articleUserGroupMapper;

    @Autowired
    private ArticleUserGroupMapperExt articleUserGroupMapperExt;

    @Autowired
    private ArticleGzhDeveloperMapper articleGzhDeveloperMapper;

    @Value("${gzh.user.group.size:8}")
    private Integer gzhUserGroupSize;

    @Value("${gzh.user.group.per.count:10000}")
    private Integer gzhUserGroupPerCount;

    public void addGZH(String gzhId, String appId, int groupNum) {

        // syncAllUserOnce(gzhId, groupNum);

        // 同步用户
        log.info("start sync user gzhId={}", gzhId);

        WxUserGetResponse response = wxUserManagementRemoteService.getAllUserLimit10000(gzhId);
        if (response == null) {
            return;
        }

        int total = response.getTotal();
        int count = response.getCount();
        log.info("sync user gzhId={} total={} count={}", gzhId, total, count);
        List<String> openIds = response.getData() == null
                ? Collections.emptyList()
                : response.getData().getOpenid();
        String nextOpenId = response.getNext_openid();
        if (CollectionUtils.isNotEmpty(openIds)) {
            List<ArticleUserGroup> userGroups = CommonCollectionUtils.toList(openIds, s -> {
                ArticleUserGroup ug = new ArticleUserGroup();
                ug.setGzhId(gzhId);
                ug.setOpenId(s);
                ug.setUserGroupId(RandomUtils.nextInt(groupNum));
                ug.setIsDelete(0);
                return ug;
            });

            //articleUserGroupRepository.saveAll(userGroups);
            articleUserGroupMapperExt.insertBatch(userGroups);
        }

        while (StringUtils.isNotBlank(nextOpenId)) {
            response = wxUserManagementRemoteService.getAllUserLimit10000(gzhId, nextOpenId);
            if (response == null) {
                return;
            }
            openIds = response.getData() == null
                    ? Collections.emptyList()
                    : response.getData().getOpenid();
            nextOpenId = response.getNext_openid();
            count += response.getCount();
            log.info("sync user gzhId={} total={} count={}", gzhId, total, count);
            if (CollectionUtils.isNotEmpty(openIds)) {
                List<ArticleUserGroup> userGroups = CommonCollectionUtils.toList(openIds, s -> {
                    ArticleUserGroup ug = new ArticleUserGroup();
                    ug.setGzhId(gzhId);
                    ug.setOpenId(s);
                    ug.setUserGroupId(RandomUtils.nextInt(groupNum));
                    ug.setIsDelete(0);
                    return ug;
                });
                articleUserGroupMapperExt.insertBatch(userGroups);
                // articleUserGroupRepository.saveAll(userGroups);
            }
        }

        log.info("end sync user gzhId={}", gzhId);

        ArticleGzhDeveloper obj = new ArticleGzhDeveloper();
        obj.setAppId(appId);
        obj.setGzhId(gzhId);
        articleGzhDeveloperMapper.insert(obj);

        resetUserGroup(gzhId);
    }

    public void resetUserGroup(String gzhId) {
        Long count = getGzhUserCount(gzhId);
        long pageSize = gzhUserGroupPerCount;
        int pageNum = 1;
        if (count < ((long) gzhUserGroupSize * gzhUserGroupPerCount)) {
            pageSize = (count / gzhUserGroupSize) + 1;
        }
        int groupId = 0;
        while (true) {
            int offset = (pageNum - 1) * (int) pageSize;
            List<ArticleUserGroup> userGroups = articleUserGroupMapperExt.selectByGzhIdAndPage(gzhId, offset, pageSize);
            if (CollectionUtils.isEmpty(userGroups)) {
                break;
            }

            pageNum++;
            List<Long> ids = CommonCollectionUtils.toList(userGroups, ArticleUserGroup::getId);
            articleUserGroupMapperExt.updateUserGroup(ids, groupId);
            groupId++;
        }
    }

//    private void syncAllUserOnce(String gzhId, int groupNum) {
//        List<String> openIds = wxUserManagementRemoteService.getAllUser(gzhId);
//        // 相对均匀
//        Collections.shuffle(openIds);
//        int size = openIds.size() / groupNum;
//        List<List<String>> partition = ListUtils.partition(openIds, size);
//        for (int i = 0; i < (openIds.size() % groupNum); i++) {
//            partition.get(i).add(openIds.get(size - i - 1));
//        }
//
//        // 写入DB
//        int ugId = 0;
//        for (List<String> partOpenIds : partition) {
//
//            int finalUgId = ugId;
//            List<ArticleUserGroup> userGroups = CommonCollectionUtils.toList(partOpenIds, s -> {
//                ArticleUserGroup ug = new ArticleUserGroup();
//                ug.setGzhId(gzhId);
//                ug.setOpenId(s);
//                ug.setUserGroupId(finalUgId);
//                return ug;
//            });
//            articleUserGroupRepository.saveAll(userGroups);
//            ugId++;
//        }
//    }

    /**
     * https://developers.weixin.qq.com/doc/offiaccount/Message_Management/Receiving_event_pushes.html
     *
     * <xml>
     * <ToUserName><![CDATA[toUser]]></ToUserName>
     * <FromUserName><![CDATA[FromUser]]></FromUserName>
     * <CreateTime>123456789</CreateTime>
     * <MsgType><![CDATA[event]]></MsgType>
     * <Event><![CDATA[subscribe]]></Event>
     * </xml>
     */
    public void listenWx(String xmlData) {
        try {
            SAXReader saxReader = new SAXReader();
            Document document = saxReader.read(new StringReader(xmlData));
            Element root = document.getRootElement();
            Map<String, String> param = new HashMap<>();
            for (Element e : root.elements()) {
                param.put(e.getName(), e.getTextTrim());
            }

            String gzhId = param.get("ToUserName");
            String openId = param.get("FromUserName");

            ArticleGzhDeveloper agd = articleGzhDeveloperMapper.selectByPrimaryKey(gzhId);
            if (agd == null) {
                // 不再体系内的账号，不记录链接
                return;
            }

            if (StringUtils.equals("event", param.get("MsgType"))) {
                String event = param.get("Event");
                switch (event) {
                    case "subscribe":
                        handleSubscribe(gzhId, openId);
                        break;
                    case "unsubscribe":
                        handleUnsubscribe(gzhId, openId);
                        break;
                    default:
                        break;
                }
            }

        } catch (Exception e) {
            log.error("listenWx handle error {} ", xmlData, e);
        }
    }

    private void handleSubscribe(String gzhId, String openId) {
        ArticleUserGroup group = articleUserGroupMapperExt.selectByGzhIdAndOpenId(gzhId, openId);
        Long count = getGzhUserCount(gzhId);
        if (group == null) {
            // insert
            // 查库，知道所有分组
            List<UserGroupCountDTO> userGroupCountDTOList = articleUserGroupMapperExt.selectUserGroupIdByGzhId(gzhId);
            // 选择分组
            int ugId = getUserGroupId(userGroupCountDTOList, count);
            group = new ArticleUserGroup();
            group.setGzhId(gzhId);
            group.setOpenId(openId);
            group.setUserGroupId(ugId);
            group.setIsDelete(0);
            articleUserGroupMapper.insert(group);
        } else {
            // update
            group.setIsDelete(0);
            articleUserGroupMapper.updateByPrimaryKey(group);
        }

    }

    private Long getGzhUserCount(String gzhId) {
        ArticleUserGroupExample countExample = new ArticleUserGroupExample();
        countExample.createCriteria().andGzhIdEqualTo(gzhId);
        return articleUserGroupMapper.countByExample(countExample);
    }

    private Integer getUserGroupId(List<UserGroupCountDTO> userGroupCountDTOList, long count) {
        if (count >= ((long) gzhUserGroupSize * gzhUserGroupPerCount)
                || userGroupCountDTOList.size() > gzhUserGroupSize) {
            userGroupCountDTOList.sort(Comparator.comparing(UserGroupCountDTO::getUserGroupId));
            UserGroupCountDTO max = userGroupCountDTOList.get(userGroupCountDTOList.size() - 1);
            if (max.getCount() < gzhUserGroupPerCount) {
                return max.getUserGroupId();
            } else {
                return max.getUserGroupId() + 1;
            }
        } else {
            int ugId = RandomUtils.nextInt(userGroupCountDTOList.size());
            while (userGroupCountDTOList.get(ugId).getCount() >= gzhUserGroupPerCount) {
                ugId = RandomUtils.nextInt(userGroupCountDTOList.size());
            }
            return ugId;
        }
    }

    private void handleUnsubscribe(String gzhId, String openId) {
        ArticleUserGroup group = articleUserGroupMapperExt.selectByGzhIdAndOpenId(gzhId, openId);
        if (group != null) {
            group.setIsDelete(1);
            articleUserGroupMapper.updateByPrimaryKey(group);
        }
    }

    public List<String> listByUserGroupId(String gzhId, int userGroupId) {
        return articleUserGroupMapperExt.selectOpenIdByGzhIdAndUserGroupId(gzhId, userGroupId);
    }
}
