package com.tzld.longarticle.recommend.server.service.recommend.rank;


import cn.hutool.core.collection.CollectionUtil;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.ctrip.framework.apollo.model.ConfigChangeEvent;
import com.ctrip.framework.apollo.spring.annotation.ApolloConfigChangeListener;
import com.ctrip.framework.apollo.spring.annotation.ApolloJsonValue;
import com.tzld.longarticle.recommend.server.common.ContentCountMonitor;
import com.tzld.longarticle.recommend.server.common.enums.aigc.PublishContentStatusEnum;
import com.tzld.longarticle.recommend.server.common.enums.aigc.PublishPlanInputSourceTypesEnum;
import com.tzld.longarticle.recommend.server.common.enums.recommend.RankStrategyEnum;
import com.tzld.longarticle.recommend.server.mapper.longArticle.LongArticleBaseMapper;
import com.tzld.longarticle.recommend.server.model.dto.Content;
import com.tzld.longarticle.recommend.server.model.entity.aigc.PublishContent;
import com.tzld.longarticle.recommend.server.repository.aigc.PublishContentRepository;
import com.tzld.longarticle.recommend.server.service.ServiceBeanFactory;
import com.tzld.longarticle.recommend.server.service.recommend.rank.strategy.FwhColdStartRankStrategy;
import com.tzld.longarticle.recommend.server.service.recommend.score.AccountIndexReplacePoolConfig;
import com.tzld.longarticle.recommend.server.util.TitleSimilarCheckUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.RandomUtils;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

import static com.tzld.longarticle.recommend.server.common.constant.SceneConstants.FWH_COLD_START;

/**
 * @author dyp
 */
@Service
@Slf4j
public class RankService implements InitializingBean {

    @Autowired
    PublishContentRepository publishContentRepository;

    @Autowired
    LongArticleBaseMapper longArticleBaseMapper;

    private static Map<String, Map<String, Integer>> staticStrategyPoolSourceTypeMap;
    private static Map<String, Double> staticSafeReduceConfig;

    @ApolloJsonValue("${strategyPoolSourceTypeConfig:{}}")
    private Map<String, Map<String, Integer>> strategyPoolSourceTypeMap;

    @ApolloJsonValue("${reduce.safe.config:{}}")
    private Map<String, Double> safeReduceConfig;

    @Override
    public void afterPropertiesSet() throws Exception {
        RankService.staticStrategyPoolSourceTypeMap = strategyPoolSourceTypeMap;
        RankService.staticSafeReduceConfig = safeReduceConfig;
    }

    @ApolloConfigChangeListener(interestedKeys = {"strategyPoolSourceTypeConfig"})
    public void configChange(ConfigChangeEvent changeEvent) {
        RankService.staticStrategyPoolSourceTypeMap = JSONObject.parseObject(changeEvent.getChange(
                "strategyPoolSourceTypeConfig").getNewValue(), Map.class);
        log.info("strategyPoolSourceTypeConfig change updateStaticValue success newValue:{}",
                JSONObject.toJSONString(staticStrategyPoolSourceTypeMap));
    }

    @ApolloConfigChangeListener(interestedKeys = {"reduce.safe.config"})
    public void configChangeSafeReduce(ConfigChangeEvent changeEvent) {
        RankService.staticSafeReduceConfig = JSONObject.parseObject(changeEvent.getChange(
                "reduce.safe.config").getNewValue(), Map.class);
        log.info("reduce.safe.config change updateStaticValue success newValue:{}",
                JSONObject.toJSONString(staticSafeReduceConfig));
    }

    public static Integer getStrategyPoolSourceType(String strategy, Integer index) {
        Integer sourceType = 0;
        Map<String, Integer> indexSourceTypeMap = staticStrategyPoolSourceTypeMap.get(strategy);
        if (Objects.nonNull(indexSourceTypeMap)) {
            sourceType = indexSourceTypeMap.get(String.valueOf(index));
        }
        return sourceType;
    }

    public static Map<String, Integer> getStrategyPoolSourceType(String strategy) {
        return staticStrategyPoolSourceTypeMap.get(strategy);
    }

    public RankResult rank(RankParam param) {
        RankStrategy strategy = getRankStrategy(param);
        return strategy.rank(param);
    }

    private RankStrategy getRankStrategy(RankParam param) {

        switch (param.getScene()) {
            case FWH_COLD_START:
                return ServiceBeanFactory.getBean(FwhColdStartRankStrategy.class);
            default:
                RankStrategyEnum rankStrategyEnum = RankStrategyEnum.from(param.getStrategy());
                return (RankStrategy) ServiceBeanFactory.getBeanByName(rankStrategyEnum.getTaskExecutorName());
        }

    }

    public static void printSortLog(String strategy, String accountName, String position, List<Content> contentList) {
        JSONArray jsonArray = new JSONArray();
        ContentCountMonitor.logCount("Rank", position, Objects.isNull(contentList) ? 0 : contentList.size());
        if (CollectionUtil.isEmpty(contentList)) {
            return;
        }
        contentList = contentList.subList(0, Math.min(contentList.size(), 200));
        for (Content content : contentList) {
            JSONObject obj = new JSONObject();
            obj.put("id", content.getId());
            obj.put("title", content.getTitle());
            obj.put("score", content.getScore());
            obj.put("scoreMap", content.getScoreMap());
            jsonArray.add(obj);
        }
        log.info("{} 账号名称 {} {}评分结果 {}", strategy, accountName, position, JSONObject.toJSONString(jsonArray));
    }

    public static void commonAddSecondContent(RankParam param, List<Content> result,
                                              String[] publishPool, String[] contentPools,
                                              Map<String, List<Content>> contentMap,
                                              Map<Integer, AccountIndexReplacePoolConfig> indexReplacePoolConfigMap,
                                              String strategy) {
        List<Content> pool = contentMap.get(contentPools[1]);
        if (CollectionUtils.isNotEmpty(pool)) {
            pool = contentSourceTypeFilter(param.getStrategy(), pool, 2);
        }
        RankService.printSortLog(strategy, param.getAccountName(), "次条", pool);
        if (CollectionUtils.isNotEmpty(pool)) {
            int i = RandomUtils.nextInt(0, Math.min(pool.size(), 5));
            int j = RandomUtils.nextInt(0, Math.min(pool.size(), 5));
            result.add(pool.get(i));
            // 替补 头条内容不足使用次条内容
            if (result.size() == 1 && pool.size() > 1) {
                while (i == j && pool.size() > 1) {
                    j = RandomUtils.nextInt(0, Math.min(pool.size(), 5));
                    if (i != j) {
                        publishPool[0] = contentPools[1];
                        result.add(pool.get(1));
                        break;
                    }
                }
            }
        } else {
            // 替补 根据设置替补内容池查找内容进行替补
            findReplacePoolContent(result, publishPool, contentMap, indexReplacePoolConfigMap);
        }
    }

    public static void findReplacePoolContent(List<Content> result, String[] publishPool,
                                              Map<String, List<Content>> contentMap,
                                              Map<Integer, AccountIndexReplacePoolConfig> indexReplacePoolConfigMap) {
        AccountIndexReplacePoolConfig replacePoolConfig = indexReplacePoolConfigMap.get(2);
        if (Objects.nonNull(replacePoolConfig)) {
            List<Content> poolReplace = contentMap.get(replacePoolConfig.getContentPool());
            if (CollectionUtils.isNotEmpty(poolReplace)) {
                publishPool[1] = replacePoolConfig.getContentPool();
                result.add(poolReplace.get(0));
            }
        }
    }

    public static List<Content> contentSourceTypeFilter(String strategy, List<Content> pool, Integer index) {
        Integer sourceType = getStrategyPoolSourceType(strategy, index);
        Integer videoSourceType = PublishPlanInputSourceTypesEnum.longArticleVideoPoolSource.getVal();
        List<Content> videoPool = pool.stream().filter(o -> Objects.equals(o.getSourceType(), videoSourceType)).collect(Collectors.toList());
        List<Content> otherPool = pool.stream().filter(o -> !Objects.equals(o.getSourceType(), videoSourceType)).collect(Collectors.toList());
        if (Objects.equals(sourceType, PublishPlanInputSourceTypesEnum.all.getVal())) {
            return pool;
        } else if (sourceType.equals(videoSourceType)) {
            if (CollectionUtils.isNotEmpty(videoPool)) {
                return videoPool;
            }
        }
        return otherPool;
    }

    public static void commonAdd38Content(RankParam param, List<Content> result, String[] contentPools,
                                          Map<String, List<Content>> contentMap, String strategy) {
        List<Content> pool = contentMap.get(contentPools[2]);
        if (CollectionUtils.isNotEmpty(pool)) {
            pool = contentSourceTypeFilter(param.getStrategy(), pool, 3);
        }

        if (CollectionUtils.isNotEmpty(pool) && param.getSize() > result.size()) {
            RankService.printSortLog(strategy, param.getAccountName(), "3-8", pool);
            result.addAll(pool.subList(0, Math.min(pool.size(), param.getSize() - result.size())));
        }
    }

    public static void safeScoreDecrease(List<RankItem> items) {
        for (RankItem item : items) {
            Content content = item.getContent();
            if (Objects.nonNull(content.getKimiSafeScore())) {
                Double safeScore = staticSafeReduceConfig.get(content.getKimiSafeScore());
                if (Objects.nonNull(safeScore)) {
                    content.setScore(content.getScore() - safeScore);
                }
            }
        }
    }

    public void checkPublishContentStatus(List<Content> result, Map<String, List<Content>> contentMap, String[] publishPool) {
        List<String> titles = result.stream().map(Content::getTitle).collect(Collectors.toList());
        List<String> publishContentIds = result.stream().map(Content::getId).collect(Collectors.toList());
        List<PublishContent> publishContentList = publishContentRepository.getByIdIn(publishContentIds);
        Map<String, Integer> publishContentStatusMap = publishContentList.stream().collect(Collectors.toMap(PublishContent::getId, PublishContent::getStatus));
        List<String> removeContentIds = new ArrayList<>();
        for (int i = 0; i < result.size(); i++) {
            if (publishContentStatusMap.get(result.get(i).getId()).equals(PublishContentStatusEnum.waiting_publish.getVal())) {
                continue;
            } else {
                removeContentIds.add(result.get(i).getId());
            }
            if (i == 0) {
                for (Content content : contentMap.get(publishPool[0])) {
                    if (!TitleSimilarCheckUtil.isDuplicateContent(content.getTitle(), titles, TitleSimilarCheckUtil.SIMILARITY_THRESHOLD)) {
                        result.set(i, content);
                        titles.add(content.getTitle());
                        break;
                    }
                }
            } else if (i == 1) {
                for (Content content : contentMap.get(publishPool[1])) {
                    if (!TitleSimilarCheckUtil.isDuplicateContent(content.getTitle(), titles, TitleSimilarCheckUtil.SIMILARITY_THRESHOLD)) {
                        result.set(i, content);
                        titles.add(content.getTitle());
                        break;
                    }
                }
            } else {
                for (Content content : contentMap.get(publishPool[2])) {
                    if (!TitleSimilarCheckUtil.isDuplicateContent(content.getTitle(), titles, TitleSimilarCheckUtil.SIMILARITY_THRESHOLD)) {
                        result.set(i, content);
                        titles.add(content.getTitle());
                        break;
                    }
                }
            }
        }
        if (CollectionUtils.isNotEmpty(removeContentIds)) {
            long updateTimestamp = System.currentTimeMillis();
            longArticleBaseMapper.updatePublishContentGzhWaitingStatusByIds(removeContentIds, 0, updateTimestamp);
        }
    }
}
