package com.tzld.longarticle.recommend.server.service.recommend.recall.strategy;

import com.tzld.longarticle.recommend.server.common.CostMonitor;
import com.tzld.longarticle.recommend.server.model.dto.Content;
import com.tzld.longarticle.recommend.server.service.recommend.filter.FilterParam;
import com.tzld.longarticle.recommend.server.service.recommend.filter.FilterResult;
import com.tzld.longarticle.recommend.server.service.recommend.filter.FilterService;
import com.tzld.longarticle.recommend.server.service.recommend.recall.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.stream.Collectors;

/**
 * @author dyp
 */
@Component
public class DefaultRecallStrategy implements RecallStrategy {

    @Autowired
    private FilterService filterService;
    @Autowired
    RecallService recallService;

    @Override
    public RecallResult.RecallData recall(RecallParam param) {
        long t1 = System.currentTimeMillis();
        List<Content> content = param.getContent();
        // 处理 content
        FilterParam filterParam = FilterParamFactory.create(param, content);
        FilterResult filterResult = filterService.filter(filterParam, false);
        long t2 = System.currentTimeMillis();
        CostMonitor.logCost("Recall", "ProcessFilter", t2 - t1);
        // 处理 content
        RecallResult.RecallData result = new RecallResult.RecallData();
        result.setContents(content.stream().filter(o -> filterResult.getContentIds().contains(o.getId()))
                .collect(Collectors.toList()));
        result.setFilterContents(filterResult.getFilterContent());
        long t3 = System.currentTimeMillis();
        CostMonitor.logCost("Recall", "SetResults", t3 - t2);
        return result;
    }

}
