package com.tzld.longarticle.recommend.server.service.video.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.ctrip.framework.apollo.spring.annotation.ApolloJsonValue;
import com.tzld.longarticle.recommend.server.model.bo.CardData;
import com.tzld.longarticle.recommend.server.service.video.CardGeneratorService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.*;

@Slf4j
@Service
public class CardGeneratorServiceImpl implements CardGeneratorService {

    @ApolloJsonValue("${miniMap:{}}")
    private JSONObject miniProgramMap;

    public CardData generateCards(String ghId, String response, Integer miniprogramUseType) {
        JSONArray jsonArray = JSONArray.parseArray(response);
        CardData cardData = new CardData();
        List<JSONObject> cardList = new ArrayList<>();
        JSONArray newItemList = new JSONArray();
        for (int i = 0; i < jsonArray.size(); i++) {
            JSONObject item = jsonArray.getJSONObject(i);
            JSONObject card = generateSingleCard(i + 1, ghId, 25, item, miniprogramUseType);
            cardList.add(card);
            newItemList.add(item);
        }
        cardData.setCardList(cardList);
        cardData.setNewItemList(newItemList);
        return cardData;

    }

    public JSONObject generateSingleCard(Integer index, String ghId, Integer miniId, JSONObject item,
                                         Integer miniprogramUseType) {
        String strMiniId = String.valueOf(miniId);
        JSONObject miniInfo = miniProgramMap.getJSONObject(strMiniId);
        String avatar = miniInfo.getString("avatar");
        String appId = miniInfo.getString("id");
        String appName = miniInfo.getString("name");
        String videoId = item.getString("videoId");
        String uid = item.getString("uid");
        JSONObject jsonObject = createGzhPath(videoId, uid, ghId, miniprogramUseType);
        String rootShareId = jsonObject.getString("rootShareId");
        String rootSourceId = jsonObject.getString("rootSourceId");
        String productionPath = jsonObject.getString("productionPath");

        JSONObject result = new JSONObject();
        result.put("productionCover", item.getString("videoCover"));
        result.put("productionName", item.getString("kimiTitle"));
        result.put("programAvatar", avatar);
        result.put("programId", appId);
        result.put("programName", appName);
        result.put("source", item.getString("source"));
        result.put("rootShareId", rootShareId);
        result.put("rootSourceId", rootSourceId);
        result.put("productionPath", productionPath);
        result.put("videoUrl", item.getString("videoPath"));
        result.put("mini_id", miniId);
        if (index == 1) {
            result.put("paragraphPosition", 0.01);
        } else {
            result.put("paragraphPosition", (index - 1) * 0.25);
        }
        item.put("rootSourceId", rootSourceId);
        return result;
    }

    public JSONObject createGzhPath(String videoId, String sharedUid, String ghId, Integer miniProgramType) {
        try {
            String rootShareId = UUID.randomUUID().toString();
            String rootSourceId;
            switch (miniProgramType) {
                case 1:
                    rootSourceId = "longArticles_" + generateSourceId();
                    break;
                case 2:
                    rootSourceId = "touliu_tencentGzhArticle_" + ghId + "_" + generateSourceId();
                    break;
                case 3:
                    rootSourceId = "WeCom_" + generateSourceId();
                    break;
                case 4:
                    rootSourceId = "DaiTou_" + ghId + "_" + generateSourceId();
                    break;
                case 5:
                    rootSourceId = "gzhhzdx_" + generateSourceId();
                    break;
                case 6:
                    rootSourceId = "fwhhzdyy_" + generateSourceId();
                    break;
                case 7:
                    rootSourceId = "fwhtouliu_" + ghId + "_" + generateSourceId();
                    break;
                case 8:
                    rootSourceId = "fwhmh_" + generateSourceId();
                    break;
                default:
                    rootSourceId = "Error mini_program_type " + miniProgramType;
            }

            String url = String.format("pages/user-videos?id=%s&su=%s&fromGzh=1&rootShareId=%s&shareId=%s&rootSourceId=%s",
                    videoId, sharedUid, rootShareId, rootShareId, rootSourceId);
            String productionPath = String.format("pages/category?jumpPage=%s", URLEncoder.encode(url, StandardCharsets.UTF_8.toString()));
            JSONObject jsonObject = new JSONObject();
            jsonObject.put("rootShareId", rootShareId);
            jsonObject.put("rootSourceId", rootSourceId);
            jsonObject.put("productionPath", productionPath);
            return jsonObject;
        } catch (UnsupportedEncodingException e) {
            log.error("createGzhPath error", e);
        }
        return new JSONObject();
    }

    private String generateSourceId() {
        long timestamp = System.currentTimeMillis();
        int randomNum = new Random().nextInt(9000) + 1000;
        String hashInput = timestamp + "-" + randomNum;
        return md5(hashInput);
    }

    private String md5(String input) {
        try {
            MessageDigest md = MessageDigest.getInstance("MD5");
            byte[] messageDigest = md.digest(input.getBytes(StandardCharsets.UTF_8));
            StringBuilder sb = new StringBuilder();
            for (byte b : messageDigest) {
                sb.append(String.format("%02x", b));
            }
            return sb.toString();
        } catch (NoSuchAlgorithmException e) {
            log.error("CardServiceImpl md5 error", e);
        }
        return null;
    }

}
