package com.tzld.longarticle.recommend.server.util;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.connection.ReturnType;
import org.springframework.data.redis.core.RedisCallback;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Component;

import java.util.concurrent.TimeUnit;

@Component
public class RedisUtil {

    @Autowired
    private RedisTemplate<String, String> redisTemplate;

    public boolean tryAcquireLock(String lockKey, String requestId) {
        // 尝试获取锁
        Boolean lockAcquired = redisTemplate.opsForValue().setIfAbsent(lockKey, requestId, 10, TimeUnit.SECONDS);
        return Boolean.TRUE.equals(lockAcquired);
    }

    public boolean tryAcquireLock(String lockKey, String requestId, Long time) {
        // 尝试获取锁
        Boolean lockAcquired = redisTemplate.opsForValue().setIfAbsent(lockKey, requestId, time, TimeUnit.SECONDS);
        return Boolean.TRUE.equals(lockAcquired);
    }

    public void releaseLock(String lockKey, String requestId) {
        // 使用 Lua 脚本确保释放锁的原子性
        String script = "if redis.call('get', KEYS[1]) == ARGV[1] then return redis.call('del', KEYS[1]) else return 0 end";
        redisTemplate.execute((RedisCallback<Object>) connection -> {
            connection.eval(script.getBytes(), ReturnType.INTEGER, 1, lockKey.getBytes(), requestId.getBytes());
            return null;
        });
    }
}
